﻿using Polly;
using Polly.Retry;
using System;
using System.Collections.Generic;
using System.Linq;
namespace PollyForNonHttpRequests
{
    class Program
    {
        static void Main(string[] args)
        {
            Program p = new Program();
            p.CheckForException();
            p.CheckForNonZeroOrException();
            p.CheckingSizeOfReturnedList();
            p.CheckingBool();
            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
        }

        private void CheckForException()
        {
            ErrorProneCode errorProneCode = new ErrorProneCode();

            RetryPolicy retryIfException = Policy.Handle<Exception>()
                .Retry(4, (exception, retryCount) =>
                {
                    Console.WriteLine($"Got a response of {exception.Message} (expected 0), retrying {retryCount}");
                });

            retryIfException.Execute(errorProneCode.DoSomethingThatMightThrowException);
        }
        private void CheckForNonZeroOrException()
        {
            ErrorProneCode errorProneCode = new ErrorProneCode();

            // Retry if the response is not 0 or there is a DivideByZeroException
            RetryPolicy<int> retryPolicyNeedsAResponseOfOne = Policy.HandleResult<int>(i => i != 0)
                .Or<DivideByZeroException>()
                .Retry(4, (response, retryCount) =>
                {
                    Console.WriteLine($"Got a response of {response.Result} (expected 0), retrying {retryCount}");
                });

            int number = retryPolicyNeedsAResponseOfOne.Execute(() => errorProneCode.GetSomeNumber());

            Console.WriteLine($"Got expected response = {number}\n\n");
        }

        private void CheckingSizeOfReturnedList()
        {
            ErrorProneCode errorProneCode = new ErrorProneCode();

            // Retry if the IEnumerable does not contain three items 
            RetryPolicy<IEnumerable<int>> retryPolicyNeedsResponseWithTwoNumbers = Policy.HandleResult<IEnumerable<int>>(l => l.Count() != 3)
               .Retry(4, (response, retryCount) =>
               {
                   Console.WriteLine($"Got a response with {response.Result.Count()} entries (expected 3), retrying {retryCount}");
               });

            var numbers = retryPolicyNeedsResponseWithTwoNumbers.Execute(() => errorProneCode.GetListOfNumbers());

            Console.WriteLine($"Got expected response of {numbers.Count()} entries\n\n");
        }

        private void CheckingBool()
        {
            ErrorProneCode errorProneCode = new ErrorProneCode();

            // Retry if the result is not true
            RetryPolicy<bool> retryPolicyNeedsTrueResponse = Policy.HandleResult<bool>(b => b != true)
               .Retry(4, (response, retryCount) =>
               {
                   Console.WriteLine($"Got a response of {response.Result} entries (expected true), retrying {retryCount}");
               });

            bool result = retryPolicyNeedsTrueResponse.Execute(() => errorProneCode.GetBool());

            Console.WriteLine($"Got expected response of {result}\n\n");
        }
    }
}
